#pragma once

#include "nsIDOMWindow.h"
#include "nsIDOMHTMLElement.h"
#include "unfrozen/include/jsapi.h"
#include "boost\noncopyable.hpp"
#include <map>
#include <string>
#include <vector>
#include <atlbase.h>

class SpiderMonkeyWrapper: boost::noncopyable
{
public:
	SpiderMonkeyWrapper(void);
	~SpiderMonkeyWrapper(void);

	bool Init(nsIDOMHTMLElement *elem,bool seperateGlobal = true,bool seperateContext = true,bool seperateRuntime = false);
	bool LoadScriptFromFile(const std::string &filename,bool execute = false);
	
	bool RunScriptFunctionFromFile(const std::string &filename,const std::string &funcName,VARIANT *retVal);

protected:

	static void OnError(JSContext *context,const char *message,JSErrorReport *report);

	//Holds the current context that is used 
	//by the SpiderMonkey engine.
	JSContext *m_context;
	//Holds the current process's runtime that is used 
	//by the SpiderMonkey engine.
	JSRuntime *m_runtime;
	//Holds the global object that will be used when running the 
	//JavaScripts.
	JSObject  *m_globalObject;

	typedef std::map<std::string,JSScript *> FilenameToCompiledScriptMap;
	typedef FilenameToCompiledScriptMap::iterator FilenameToCompiledScriptMapItr;
	FilenameToCompiledScriptMap m_fileToScript;

private:
	nsIDOMWindow *GetWindowlFromElement(nsIDOMHTMLElement *elem);

	bool m_destroyRuntime;
	bool m_destroyContext;
};
